C	FILE BspcRW.FOR	Version 1
C     Written by John Bertie, University of Alberta, Edmonton, Canada
C     November, 2001
C
C   NOTE THAT READSC1 and READSC2 check that the number of points in 
C   the file does not exceed NPTLIMIT, which is passed to them in 
C   COMMON/FOR_READ/NPTLIMIT.  If the number exceeds NPTLIMIT they do not read
C   the file but print a statement and RETURN

C  
C     Written for compilation with Digital Fortran version 5.0A, which
C     is the successor to Microsoft FORTRAN Powerstation version 4 and
C     in turn has been succeeded by Compaq FORTRAN.
C     Digital Fortran permits use of FORTRAN 90 and parts of
C     FORTRAN 95.  However, the programs are largely written for
C     compatibility with Microsoft FORTRAN 5.1, and largely use 
C     FORTRAN 77 with some extensions.  The main known extensions in
C     these subroutines are the use of "FORM=Binary" in the "OPEN"
C     statements and the INTEGER*1 and INTEGER*2 variable types.
C
C  This file contains subroutines to determine whether a file is of 
C  the old (type 1) or the new (type 2) SPC format then read the file;
C  also to write files of both types as requested by the user through
C  parameter FVERSN.
C  In the case of type 2 files, the routine READSC also sets some
C  variables to the types and names used for the type 1 files, in
C  order to avoid changes in existing programs that use these routines.
C 
C  Note that only single spectrum files are considered, although the
C  SPC type 2 format allows multi-spectrum files to be created,
C  as is described in the Galactic Universal Data Format Specification
C  In Galactic's SPC File Developer's Kit
C  (see http://www.galactic.com/instruments/spc.htm)
C
C		*************************************
C
C  This file contains the subroutines READSC,READSC1,READSC2,WRITESC,
C  WRITESC1,WRITESC2,GETNAM,LENSTR,MSG,AHILO,GETCHAN,PUTCHAN,GSPCNAM
C  and LENSTR1

C   Based on Van Every's RWSPC for the old SPC files.


	SUBROUTINE READSC(inspec,R,NP,XS,XE,XT,YT,RES,COMM,IER)
C  Subroutine to open the file and read the first two bytes, determine
C  whether it is an old (type 1) or new (type 2) SPC file, and call
C  the appropriate subroutine to read the file.  In the case of type 2
C  files it also converts the variable names in these new files to the
C  old names for consistency with existing programs.

	CHARACTER*1     New
      CHARACTER*9     fres
	CHARACTER*12    inspec
      CHARACTER*130   COMM,fcmnt
	INTEGER*4       NP,ier,fnpts
	INTEGER*2       ICHAN
      INTEGER*1       ftflags, fversn, XT, YT,fxtype,fytype
      REAL*4          R(*),RES
	REAL*8          ffirst,flast

	COMMON/FOR_READ/NPTLIMIT

	COMMON/Original/New

	COMMON/SPCtype/fversn,ftflags

	COMMON/SPC2s/ffirst,flast,fnpts,fxtype,fytype

	COMMON/SPCchar/fres,fcmnt

	IER=0
C   Initialize New to New='Y'.  After READSC2 reads a file 
C   New is reset to'N'
	NEW='Y'
C   *** OPEN THE FILE
      CALL GETCHAN(ICHAN,IER)
      OPEN(UNIT=ICHAN,FILE=inspec,FORM='BINARY',
     &STATUS='UNKNOWN',ERR=9999)

C       *** READ THE first two bytes      
      READ(ICHAN) ftflags,fversn
	IF (ftflags.NE.0) THEN
  	  WRITE(*,*)'Routine READSC; File ',inspec,':ftflags is not zero.'
        CALL MSG('\READSC can not read this file and\')
	  CALL MSG('\program stops abnormally\')
	  CALL PUTCHAN(ICHAN,IER)
	  STOP 

	ELSEIF((fversn.NE.77).and.(fversn.NE.75)) THEN 
        WRITE(*,*)' fversn is neither 77 (old SPC) nor 75 (new SPC)'
	  CALL MSG('\READSC can not read this file and\')
	  CALL MSG('\ program stops abnormally\')
	  CALL PUTCHAN(ICHAN,IER)
	  STOP

	ELSEIF(fversn.EQ.77)THEN
	  CALL READSC1(inspec,ICHAN,R,NP,XS,XE,XT,YT,RES,COMM,IER)
	  CALL PUTCHAN(ICHAN,IER)
	  GoTo 999

	ELSEIF(fversn.EQ.75)THEN
	  CALL READSC2(inspec,ICHAN,R,IER)
	  CALL PUTCHAN(ICHAN,IER)

C     The following 7 statements provide compatibility with calling
C     programs written before November 2001
	  NP=fnpts
	  XS=ffirst
	  XE=flast
	  XT=fxtype
	  YT=fytype
	  READ(fres,'(F4.1,I1)',ERR=100)RES
  100	  COMM=fcmnt
 	  GOTO 999

	ENDIF
  999	RETURN
 9999 STOP 'ERROR OPENING SPECTRA-CALC FILE IN READSC'
	END


      SUBROUTINE READSC1(inspec,ICHAN,R,NP,XS,XE,XT,YT,RES,COMM,IER)
C
C   READSC1 READS A FILE in the old SPC format of GALACTIC Industries
C   program SPECTRACALC. IT IS A GENERAL PURPOSE ROUTINE SO THAT
C   DATA IN THIS FORM MAY BE ACCESSED AND MANIPULATED
C   WITH FORTRAN.
C   Originally WRITTEN in the 1980s BY KENNETH W. VAN EVERY, 
C   HIMONT R&D CENTER, 800 GREENBANK RD., WILMINGTON, DE 19808
C***********************************************************
C   PASSED ARGUMENTS
C
C       NAME    TYPE    USAGE
C       ----    ----    ------------------------------------
C       INSPEC  CHAR*12 NAME OF THE SPECTRAL FILE
C       R       REAL*4  ARRAY   REAL ARRAY WITH Y DATA
C       NP      INT*4   NUMBER OF POINTS OF Y DATA
C       XS      REAL*4  FLOATING POINT VALUE- FIRST X
C       XE      REAL*4  FLOATING POINT VALUE- LAST X
C       XT      INT*1   X DATA TYPE (SEE SC MANUAL)
C       YT      INT*1   Y DATA TYPE (SEE SC MANUAL)
C       RES     REAL*4  NOMINAL DATA RESOLUTION
C       COMM    CHAR*130 ARRAY WITH 130 BYTE COMMENT
C       BLANKS  CHAR*62  ARRAY FOR THE LAST 62 BLANK BYTES OF THE HEADER
C       IER     INT*4     ERROR CODE  0=NO ERROR
C***********************************************************
C   NOTE READSC AND WRITESC WERE CHANGED ON OCT. 25/90 BY C.D. KEEFE
C   TO HAVE RES INSTEAD OF RESO PASSED TO AND FROM THE CALLING PROGRAM.
C   ROUTINES CREATED BEFORE THIS DATE BUT RELINKED AFTER MAY NEED
C   TO BE CHANGED BECAUSE OF THIS.
C
C   *** INITIALIZE

      CHARACTER*130   COMM
      CHARACTER*62    BLANKS
      CHARACTER*12    inspec
      CHARACTER*8     RESO
      INTEGER*1       I41(4), IMON, IDAY, IHR, IMIN
      INTEGER*1       XT, YT
      INTEGER*2       ISCALE,IYR,ichan
      INTEGER*4       NP,RESERVE(8),MAXINT,I4,I
      REAL*4          R(*),RES, RNP, FACTOR

      EQUIVALENCE  (I4,I41(1))
      COMMON/DATE/IYR,IMON,IDAY,IHR,IMIN
	COMMON/FOR_READ/NPTLIMIT

C   MAXINT is 7FFFFFFF, the largest possible 32-bit signed integer
      MAXINT=2147483647
      IER=0

      READ(ICHAN) ISCALE, RNP, XS, XE, XT, YT
      NP=INT(RNP)

C   Check that the NP does not exceed NPTLIMIT

	IF(NP.GT.NPTLIMIT) THEN
	   write(*,915)inspec,NPTLIMIT
  915	   Format(' Routine READSC1: Number of Points in file ',A12,
     1' exceeds array size ',I5,/,	' File not read')	
		GO TO 25
	ENDIF

C   *** READ THE DATE
      READ(ICHAN) IYR,IMON,IDAY,IHR
      READ(ICHAN) IMIN

C   *** RESOLUTION AND RESERVED BYTE, COMMENT and 
C	the last 62 (Blank) entries of the header
      READ(ICHAN) RESO,RESERVE,COMM,BLANKS
      READ(RESO,'(F4.1,I1)',ERR=8001)RES

C   *** READ THE Y DATA
C   DESCALE AND STORE THE NUMBERS

8001  FACTOR=((MAXINT/0.5)/(2.0**ISCALE))

      DO 20 I=1,NP
      READ(ICHAN) I41(3),I41(4),I41(1),I41(2)
C   I4= R(I)*FACTOR
      R(I)=REAL(I4)/FACTOR
   20 CONTINUE
   25 RETURN
      END


      SUBROUTINE READSC2(inspec,ICHAN,R,IER)

C  READSC2 reads an SPC file of Galactic Industries latest type,
C  as defined in "Galactic Universal Data Format Specification",9/3/97,
C  but limited to the simple case of a single file. 
C  Written by John Bertie,Department of Chemistry,University of Alberta
C  Edmonton, Alberta, Canada, John.Bertie@UAlberta.Ca, to allow simple
C  infrared *.SPC files to be read whether they are in the old or new
C  *.SPC format.
C
C  All of the variables in the file are read and stored in COMMON
C  to allow them to be written to an output "New" SPC file if wanted.

C  inspec is the CHARACTER*12 name of the spectral file
C***************************************************************
C		VARIABLES
C Byte 1     ftflgs  INTEGER*1  File type flag,BSPC only handles zero
C Byte 2     fversn  INTEGER*1  SPC version indicator,
C 		 				     value 77-type 1, 75-type 2
C Byte 3     fexper  INTEGER*1  Experiment type flag.  Set to 0 (zero)
C Byte 4     fexp    INTEGER*1  Y scale exponent of 2.
C                               Y values are REAL*4 if fexp=-128 (80h)
C Byte 5-8   fnpts   INTEGER*4  Number of Y values
C Byte 9-16  ffirst  REAL*8     First X value
C Byte 17-24 flast   REAL*8     Last X value
C Byte 25-28 fnsub   INTEGER*4  Number of sub files.  BSPC handles 1
C Byte 29    fxtype  INTEGER*1  Type of x unit:1-wavenumber
C Byte 30    fytype  INTEGER*1  Type of Y unit:0=arbitrary,1=absorbance
C Byte 31    fztype  INTEGER*1  Type of z unit; set to 0
C Byte 32    fpost   INTEGER*1  Posting disposition; set to 0
C Byte 33-36 fdate   INTEGER*4  YEAR(12b)mon(4b)day(5b)hour(5b)min(6b)
C Byte 37-45 fres    CHARACTER*9 Text description of resolution, NT
C                     where NT means terminated by first null character
C Byte 46-54 fsource CHARACTER*9 Instrument name or model, NT
C Byte 55-56 fpeakpt INTEGER*2   Peak point oif interferogram; set to 0
C Byte 57-88 fspare  INTEGER*4   Set 8 elements to zero
C Byte 89-218 fcmnt  CHARACTER*130  Text string describing data. NT
C Byte 219-248 fcatxt CHARACTER*30 Custom axis labels.  Set to null
C Byte 249-252 flogoff INTEGER*4  null or pointer to start of log data.
C Byte 253-256 fmods   INTEGER*4  Essentially obsolete; Usually 0
C Byte 257     fprocs  INTEGER*1  Set to zero
C Byte 258     flevel  INTEGER*1  Set to zero
C Byte 259-260 fsampin INTEGER*2  Set to zero
C Byte 261-264 ffactor INTEGER*4  Set to zero
C Byte 265-312 fmethod CHARACTER*48  Set to null
C Byte 313-316 fzinc    REAL*4     Z increment; BSPC only handles zero
C Byte 317-320 fwplanes INTEGER*4 Set to zero
C Byte 321-324 fwinc    REAL*4     W increment; BSPC only handles zero
C Byte 325     fwtype   INTEGER*1  w axis unit type; set to zero
C Byte 326-512 freserv CHARACTER*187  Set to zero
C Bytes 513 to 544 SUBHDR(32) INTEGER*1 array. 
C SUBHDR(2) the same as fexp; all others all zero for IR files.
C Bytes 545 to byte (544+fnpts*4  i.e., to byte flogoff)  Y data
C IF flogoff is not zero, the following
C Byte flogoff +1 Start of log data block
C Byte flogoff+1 Logsizd INTEGER*4 Size of log data block,text+header
C                        except from OPUS size of text log data block
C Byte flogoff+5 Logsizm INTEGER*4 Size of log data block rounded up
C                                  to next higher multiple of 4096
C Byte flogoff+9 Logtxto INTEGER*4 Offset from start of LOGSTC to
C                                  start of text log data block
C Byte flogoff+13 Logbins INTEGER*4 Set to zero
C Byte flogoff+17 logpbins INTEGER*4 Set to zero
C Byte flogoff+21 Logspar(logtxto-20) CHARACTER*1
C Byte flogoff+logtxto   LOGtext(logsizd-logtxto) CHARACTER*1 

C       INITIALIZE

	INTEGER*1 ftflags,fversn,fexper,fexp,fxtype,fytype,fztype,fpost
	INTEGER*1 fprocs,flevel,fwtype,IMON,IDAY,IHR,IMIN,SUBHDR
	INTEGER*2 fpeakpt, fsampin,ichan,IYR
	INTEGER*4 fnpts,fnsub,fdate,fspare,flogoff,fmods,ffactor
	INTEGER*4 fwplanes,I4,logsizd, logsizm,logtxto,logbins,logpbins
	INTEGER*4 NLGSPAR,NLGDATA,IER
	REAL*4    fzinc,fwinc,R(*)
	REAL*8    ffirst,flast
	CHARACTER*1 logspar,LOGtext,New,D
	CHARACTER*9 fres,fsource
	CHARACTER*12 inspec
	CHARACTER*30 fcatxt
	CHARACTER*48 fmethod
	CHARACTER*130 fcmnt
	CHARACTER*187 freserv

	DIMENSION fspare(8),logspar(256), LOGtext(1000),SUBHDR(32)

	COMMON/FOR_READ/NPTLIMIT

	COMMON/SPCtype/fversn,ftflags,fexper,fexp,fztype,fpost,
     1fprocs,flevel,fwtype,SUBHDR

	COMMON/SPC2/fzinc,fwinc,fnsub,fdate,fspare,flogoff,fmods,
     1ffactor,fwplanes,fpeakpt,fsampin,I4,logsizd,logsizm,logtxto,
     2logbins,logpbins,NLGSPAR,NLGDATA	

	COMMON/DATE/IYR,IMON,IDAY,IHR,IMIN
	
	COMMON/SPC2s/ffirst,flast,fnpts,fxtype,fytype

	COMMON/SPCchar/fres,fcmnt,freserv,fmethod,fcatxt,fsource,
     1logspar,LOGtext

	COMMON/Original/New

C   MAXINT is written as the equivalent #7FFFFFFF in RWSPC
      MAXINT=2147483647
      IER=0

      READ(ICHAN)fexper,fexp,fnpts,ffirst,flast,fnsub,fxtype,fytype

C   Check that fnpts does not exceed NPTLIMIT and return if it does

	IF(fnpts.GT.NPTLIMIT) THEN
	   write(*,915)inspec,NPTLIMIT
  915	   Format(' Routine READSC2: Number of Points in file ',A12,
     1' exceeds array size ',I5,	' File not read',/)	
		GO TO 999
	ENDIF
C  Set NEW='N' so that the subsequent write knows that a new SPC file
C  has been read and can output some of the input data if wanted. 
	New='N'
	READ(ICHAN)fztype,fpost,fdate,fres,fsource,fpeakpt
	READ(ICHAN)(fspare(I),I=1,8)
	READ(ICHAN)fcmnt,fcatxt,flogoff,fmods,fprocs,flevel,fsampin
	READ(ICHAN)ffactor,fmethod,fzinc,fwplanes,fwinc,fwtype,freserv
	READ(ICHAN)SUBHDR

C	Separate fdate into IYR, IMON, IDAY, IHR,IMIN
C     Set D='S' to Separate fdate
	D='S'
	CALL COMBDATE(fdate,IYR,IMON,IDAY,IHR,IMIN,D)

C     Read the Y values.  
C     If fexp is 80h=+128 or -128,the Y values are read as REAL*4
C     numbers.
C     If fexp is not 128 the Y values are read as 32 bit signed
C     integers,I4, such that the Real Y values,R(I), are given by
C     R=(REAL(I4))/FACTOR, where FACTOR=(2**32)/(2**fexp)
	
	IF((fexp.EQ.128).OR.(fexp.eq.-128))THEN
	   DO 20 I=1,fnpts
	   READ(ICHAN) R(I)
  20	   CONTINUE
	 CONTINUE
	ELSE	
	   FACTOR=((MAXINT/0.5)/(2.0**fexp))
         DO 22 I=1,fnpts
         READ(ICHAN) I4
         R(I)=REAL(I4)/FACTOR
  22       CONTINUE
      ENDIF

C     If flogoff is not zero, read the Log block.  This often contains
C     user-written information about the instrument and parameters
	IF(flogoff.NE.0)THEN
	  READ(ICHAN)LOGSIZD,LOGSIZM,LOGTXTO,LOGBINS,logpbins
	  NLGSPAR=LOGTXTO-20
C     The size of LOGSPAR is not defined. I have dimensioned it at
C     256 and included the following check.
	  IF(NLGSPAR.GT.256)THEN
	      WRITE(*,*)' File ',inspec
	      CALL MSG('\ NLGSPAR  larger than 256\')
	      CALL MSG('\ LOG Block not read\')
	      flogoff=0
	      GO TO 999
	  ELSE
	      DO 25 I=1,NLGSPAR
	      READ(ICHAN) LOGSPAR(I)
   25       CONTINUE
	  ENDIF
C     The size of LOGtext is not defined. I have dimensioned it at
C     1000 and included the following check.

	  	  IF((LOGSIZD+1).GT.1000)THEN
	      WRITE(*,*)' File ',inspec
	      CALL MSG('\ NLGDATA larger than 1000\')
	      CALL MSG('\ LOG Block not read\')
	      flogoff=0
	      GO TO 999
	  ELSE
C    According to Galactic's 1997 Universal Data Specification (UDF)
C    LOGSIZD is meant to be the size of the entire LOG BLOCK, i.e.
C    from its start at offset flogoff to the end of the file.
C    Some instruments, certainly Bruker's OPUS, have used it as the
C    (smaller) size of just the LOG text block instead.  
C    To cover both situations I assume it may have the latter 
C    meaning, add 1 to it to ensure that the read reaches the end of
C    file, and then adjust LOGSIZD to the Galactic UDF meaning.
	      DO 30 J=1,LOGSIZD+1
	      NLGDATA=J
	      READ(ICHAN,END=35)LOGtext(J)
   30       CONTINUE
   35       NLGDATA=NLGDATA-1
            LOGSIZD=NLGDATA+LOGTXTO
	  ENDIF
	ENDIF

  999	RETURN
	END

      SUBROUTINE WRITESC(outspec,R,NP,XS,XE,XT,YT,RES,COMM,IER)
C  Subroutine to determine which type of SPC file is wanted and open
C  the appropriate write routine

      CHARACTER*1     New,NULL,SP
	CHARACTER*9     fres
      CHARACTER*12    outspec
      CHARACTER*130   COMM,fcmnt
      INTEGER*4       NP,IER,fnpts
      INTEGER*1       ftflags, fversn,fxtype,fytype,XT,YT
      REAL*4          R(*),RES
      REAL*8          ffirst,flast

	COMMON/SPCtype/fversn,ftflags

	COMMON/SPC2s/ffirst,flast,fnpts,fxtype,fytype

	COMMON/SPCchar/fres,fcmnt

	COMMON/Original/New

C  Set SP = CHAR(32) for space and null=CHAR(0)
	SP=CHAR(32)
	NULL=CHAR(0)
C   Fill COMM with nulls after the end of the text.
	DO 20,I=1,127
	IF((COMM(I:I).EQ.SP).and.(COMM(I+1:I+1).EQ.SP).AND.(COMM(I+2:
     &I+2).EQ.SP))THEN
          M=I
		GOTO 25
	ENDIF
   20 CONTINUE
	
   25 DO 30, I=M,130
         COMM(I:I)=NULL
   30 CONTINUE

      IF(fversn.EQ.75) then
          fnpts=NP
          ffirst=XS
          flast=XE
          fxtype=XT
          fytype=YT          
          fcmnt=COMM
          CALL WRITESC2(outspec,R,IER)
          GOTO 999
      ELSE
	    WRITE(fres,'(F4.1,I1)')RES,0
          CALL WRITESC1(outspec,R,NP,XS,XE,XT,YT,RES,COMM,IER)
          ENDIF

  999 RETURN
      END

      
      SUBROUTINE WRITESC1(outspec,R,NP,XS,XE,XT,YT,RES,COMM,IER)
C
C   WRITESC WRITES A SPECTRA-CALC(R) (GALACTIC IND.)
C   FILE FORMAT. IT IS A GENERAL PURPOSE ROUTINE SO THAT
C   DATA OBTAINED FROM ANY SOURCE MAY BE MANIPULATED
C   WITH SC.
C   WRITTEN BY      KENNETH W. VAN EVERY, HIMONT R&D CENTER
C                   800 GREENBANK RD., WILMINGTON, DE 19808
C                   (302) 996-5076
C***********************************************************
C       PASSED ARGUMENTS
C
C       NAME    TYPE    USAGE
C       ----    ----    ------------------------------------
C       outspec    CHAR*12   THE FILENAME TO WRITE ON DISK
C       R       REAL*4    ARRAY   REAL ARRAY WITH Y DATA
C       NP      INT*4     NUMBER OF POINTS OF Y DATA
C       XS      REAL*4    FLOATING POINT VALUE- FIRST X
C       XE      REAL*4    FLOATING POINT VALUE- LAST X
C       XT      INT*1     X DATA TYPE (SEE SC MANUAL)
C       YT      INT*1     Y DATA TYPE (SEE SC MANUAL)
C       RES     REAL*4    NOMINAL DATA RESOLUTION
C       COMM    CHAR*130  ARRAY WITH 130 BYTE COMMENT
C       BLANKS  CHAR*62   ARRAY FOR THE LAST 62 BLANK BYTES OF THE HEADER
C       IER     INT*4     ERROR CODE  0=NO ERROR
C***********************************************************
C   NOTE READSC AND WRITESC WHERE CHANGED ON OCT. 25/90 BY C.D. KEEFE
C   TO HAVE RES INSTEAD OF RESO PASSED TO AND FROM THE CALLING PROGRAM.
C   ROUTINES CREATED BEFORE THIS DATE BUT RELINKED AFTER MAY NEED
C   TO BE CHANGED BECAUSE OF THIS.
C
C   *** INITIALIZE

      CHARACTER*12    outspec
      CHARACTER*130   COMM
      CHARACTER*62    BLANKS
      CHARACTER*8     RESO

      INTEGER*1       I41(4),imon,iday,ihr,imin,ZEROS(62)
      INTEGER*1       VER, D4, XT, YT
      INTEGER*2       ISCALE,ichan,iyr
      INTEGER*4       NP,RESERVE(8),MAXINT,I4,IER,lz
      REAL*4            R(*), FACTOR
      EQUIVALENCE  (I4,I41(1)),(BLANKS,ZEROS(1))

      COMMON/DATE/IYR,IMON,IDAY,IHR,IMIN
C
C   MAXINT is 2**31
      MAXINT=2147483647
      IER=0
C      Zero the 62 entries in BLANKS, via ZEROS
	  DO 66 I=1,62
	    ZEROS(I)=0
   66  CONTINUE

C   *** OPEN THE FILE
      CALL GETCHAN(ICHAN,IER)
      OPEN(UNIT=ICHAN,FILE=outspec,FORM='BINARY',
     &  STATUS='UNKNOWN',ERR=9999)


C   *** WRITE THE PREAMBLE
      VER=0
C   D4 is #4D hexadecimal = 77 decimal for the original SPC files
      D4=77
      WRITE(ICHAN) VER,D4

C   *** DETERMINE SCALING EXPONENT
      lz=1
      CALL AHILO(R,lz,NP,RMAX,RMIN)
C   CASE RMAX > .5
      RMAX=ABS(RMAX)
      IF( ABS(RMIN) .GT. RMAX ) RMAX=ABS(RMIN)
C   WRITE(*,*)RMAX
      IF( RMAX .GE. .5 ) THEN
            ISCALE=0
1           IF( RMAX/(2.0**ISCALE) .LT. .5 ) GO TO 50
            ISCALE=ISCALE+1
            GO TO 1

      ELSEIF( RMAX .GT. 0. )  THEN
            ISCALE=0
2           IF( RMAX/(2.0**(ISCALE-1.0)) .GT. 0.5 ) GO TO 50
C   WRITE(*,*) ' SCALE FACTOR IS ', ISCALE
            ISCALE=ISCALE-1
            GO TO 2

      ELSE
      STOP 'WRITESC: NO VALUE GT ZERO'
      ENDIF
C  50 WRITE(*,*) ' SCALE FACTOR IS ', ISCALE
50    CONTINUE

C   ***
      WRITE(ICHAN) ISCALE, REAL(NP), XS, XE, XT, YT

C   *** WRITE THE DATE
      CALL GETTIM(IHR,IMIN,ISEC,I100TH)
      CALL GETDAT(IYR,IMON,IDAY)
      WRITE(ICHAN)IYR,IMON,IDAY,IHR,imin

C   *** RESOLUTION AND RESERVED BYTE, COMMENT

C Next 2 lines are needed to avoid Compiler WARNING message
      DO 1111 LZ=1,8
1111    RESERVE(LZ)=0.

      WRITE(RESO,'(F4.1,I1)')RES,0
      WRITE(ICHAN) RESO,RESERVE,COMM,BLANKS

C   *** WRITE THE Y DATA
C   DESCALE AND STORE THE NUMBERS

      FACTOR=(MAXINT/.5)/(2.0**ISCALE)
      DO 20 I=1,NP
      I4= R(I)*FACTOR
C   WRITE(*,'(Z)') I4
      WRITE(ICHAN) I41(3),I41(4),I41(1),I41(2)
20    CONTINUE

C   *** FINISH UP AND RETURN NORMAL
      CALL PUTCHAN(ICHAN,IER)
      RETURN
C   ERROR HANDLING AND ABNORMAL RETURNS
9999    STOP 'ERROR OPENING SPECTRA-CALC FILE IN WRITESC1'
      END


	SUBROUTINE WRITESC2(outspec,R,IER)	
C  WRITESC2 writes an SPC file of Galactic Industries latest type,
C  as defined in "Galactic Universal Data Format Specification",9/3/97,
C  but limited to the simple case of a single file and limited to
C  writing the Y values as IEEE REAL*4 numbers. 
C  Written by John Bertie,Department of Chemistry,University of Alberta
C  Edmonton, Alberta, Canada, John.Bertie@UAlberta.Ca, to allow simple
C  infrared *.SPC files to be written whether they are in the old
C  or new *.SPC format.
C   outspec is the CHAR*12 name of the spectral file
C
C  The variables in the file that are not passed in from the calling program
C  are either taken from a type 2 SPC file that has been read by READSC2
C  and stored in COMMON (if NEW="N') or are initialized by the routine
C  following the Universal Data Format (if NEW='Y').
C
C  The calling program may reject any saved material and have the
C  output file initialized below by setting New='Y'.  
C  New is CHARACTER*1 and is passed in COMMON/original/New
C
C  If an SPC file is being created from an ASCII or other non-SPC file,
C  New=Y has to be set by the calling program.

C  If an SPC file is being created from an SPC file of the old type,
C  New='Y' is set by READSC and an output UDF SPC file will be
C  initialized below.

C  outspec is the CHARACTER*12 name of the spectral file
C***************************************************************
C		VARIABLES
C Byte 1     ftflgs  INTEGER*1  File type flag,BSPC only handles zero
C Byte 2     fversn  INTEGER*1  SPC version indicator,
C 		 				     value 77-type 1, 75-type 2
C Byte 3     fexper  INTEGER*1  Experiment type flag.  Set to 0 (zero)
C Byte 4     fexp    INTEGER*1  Y scale exponent of 2.
C                               Y values are REAL*4 if fexp=-128 (80h)
C Byte 5-8   fnpts   INTEGER*4  Number of Y values
C Byte 9-16  ffirst  REAL*8     First X value
C Byte 17-24 flast   REAL*8     Last X value
C Byte 25-28 fnsub   INTEGER*4  Number of sub files.  BSPC handles 1
C Byte 29    fxtype  INTEGER*1  Type of x unit:1-wavenumber
C Byte 30    fytype  INTEGER*1  Type of Y unit:0=arbitrary,1=absorbance
C Byte 31    fztype  INTEGER*1  Type of z unit; set to 0
C Byte 32    fpost   INTEGER*1  Posting disposition; set to 0
C Byte 33-36 fdate   INTEGER*4  YEAR(12b)mon(4b)day(5b)hour(5b)min(6b)
C Byte 37-45 fres    CHARACTER*9 Text description of resolution, NT
C                     where NT means terminated by first null character
C Byte 46-54 fsource CHARACTER*9 Instrument name or model, NT
C Byte 55-56 fpeakpt INTEGER*2   Peak point oif interferogram; set to 0
C Byte 57-88 fspare  INTEGER*4   Set 8 elements to zero
C Byte 89-218 fcmnt  CHARACTER*130  Text string describing data. NT
C Byte 219-248 fcatxt CHARACTER*30 Custom axis labels.  Set to null
C Byte 249-252 flogoff INTEGER*4  null or pointer to start of log data.
C Byte 253-256 fmods   INTEGER*4  Essentially obsolete; Usually 0
C Byte 257     fprocs  INTEGER*1  Set to zero
C Byte 258     flevel  INTEGER*1  Set to zero
C Byte 259-260 fsampin INTEGER*2  Set to zero
C Byte 261-264 ffactor INTEGER*4  Set to zero
C Byte 265-312 fmethod CHARACTER*48  Set to null
C Byte 313-316 fzinc    REAL*4     Z increment; BSPC only handles zero
C Byte 317-320 fwplanes INTEGER*4 Set to zero
C Byte 321-324 fwinc    REAL*4     W increment; BSPC only handles zero
C Byte 325     fwtype   INTEGER*1  w axis unit type; set to zero
C Byte 326-512 freserv CHARACTER*187  Set to zero
C Bytes 513 to 544 SUBHDR(32) INTEGER*1 array. 
C SUBHDR(2) the same as fexp; all others all zero for IR files.
C Bytes 545 to byte (544+fnpts*4  i.e., to byte flogoff)  Y data
C  IF flogoff is not zero, the following
C Byte flogoff +1 Start of log data block
C Byte flogoff+1 Logsizd INTEGER*4 Size of log data block,text+header
C                        except from OPUS size of text log data block
C Byte flogoff+5 Logsizm INTEGER*4 Size of log data block rounded up
C                                  to next higher multiple of 4096
C Byte flogoff+9 Logtxto INTEGER*4 Offset from start of LOGSTC to
C                                  start of text log data block
C Byte flogoff+13 Logbins INTEGER*4 Set to zero
C Byte flogoff+17 logpbins INTEGER*4 Set to zero
C Byte flogoff+21 Logspar(logtxto-20) CHARACTER*1
C Byte flogoff+logtxto   LOGtext(logsizd-logtxto) CHARACTER*1 

C       INITIALIZE
	INTEGER*1 ftflags,fversn,fexper,fexp,fxtype,fytype,fztype,fpost
	INTEGER*1 fprocs,flevel,fwtype,SUBHDR,IMON,IDAY,IHR,IMIN
	INTEGER*2 fpeakpt, fsampin,ichan,IYR
	INTEGER*4 fnpts,fnsub,fdate,fspare,flogoff,fmods,ffactor
	INTEGER*4 fwplanes,I4,logsizd, logsizm,logtxto,logbins,logpbins
	INTEGER*4 NLGSPAR,NLGDATA,IER
	REAL*4    fzinc,fwinc,R(*)
	REAL*8    ffirst,flast
	CHARACTER*1 logspar,LOGtext,New,NULL,SP,D
	CHARACTER*9 fres,fsource
	CHARACTER*12 outspec
	CHARACTER*30 fcatxt
	CHARACTER*48 fmethod
	CHARACTER*130 fcmnt
	CHARACTER*187 freserv

	DIMENSION fspare(8),logspar(256),LOGtext(1000),SUBHDR(32)

	COMMON/DATE/IYR,IMON,IDAY,IHR,IMIN
	
	COMMON/SPCtype/fversn,ftflags,fexper,fexp,fztype,fpost,
     1fprocs,flevel,fwtype,SUBHDR

	COMMON/SPC2/fzinc,fwinc,fnsub,fdate,fspare,flogoff,fmods,
     1ffactor,fwplanes,fpeakpt,fsampin,I4,logsizd,logsizm,logtxto,
     2logbins,logpbins,NLGSPAR,NLGDATA	

	COMMON/SPC2s/ffirst,flast,fnpts,fxtype,fytype

	COMMON/SPCchar/fres,fcmnt,freserv,fmethod,fcatxt,fsource,
     1logspar, LOGtext
	COMMON/original/New

C  Set SP = CHAR(32) for space and null=CHAR(0)
	SP=CHAR(32)
	NULL=CHAR(0)
C     Set fexp to 80 hexadecimal=-128 and write Y values as REAL*4
C     Also set SUBHDR(2) to the same as fexp
	fexp=-128
	SUBHDR(2)=fexp

C   If this is an original type 2 SPC file, not one that has been read
C   and modified arithmetically or extended, set the various parameters.
	IF((NEW.EQ.'Y').OR.(New.EQ.'y'))THEN
	    ftflags=0
	    fversn=75
	    fexper=0
	    fnsub=1
	    fztype=0
	    fpost=0
C	Create fdate
          CALL GETTIM(IHR,IMIN,ISEC,I100TH)
          CALL GETDAT(IYR,IMON,IDAY)
C     Set D='C' to create fdate
	    D='C'
	    CALL COMBDATE(fdate,IYR,IMON,IDAY,IHR,IMIN,D)
C	Set fres and fsource to all nulls
	    DO 10 I=1,9
	      fres(I:I)=NULL
	      fsource(I:I)=NULL
   10     CONTINUE
	    fpeakpt=0
	    DO 15 I=1,8
	      fspare(I)=0
   15     CONTINUE
C   SET fcatxt to NULLs
	    DO 35, I=1,30
	       fcatxt(I:I)=NULL
   35     CONTINUE
	    flogoff=0
	    fmods=0
	    fprocs=0
	    flevel=0
	    fsampin=0
	    ffactor=0
	    DO 40, I=1,48
	       fmethod(I:I)=NULL
   40     CONTINUE
	    fzinc=0
	    fwplanes=0
	    fwinc=0
	    fwtype=0
	    SUBHDR(1)=0
	    Do 45, I=3,32
	       SUBHDR(I)=0
   45     CONTINUE
	ENDIF

C  Open the output file and write it
      CALL GETCHAN(ICHAN,IER)
      OPEN(UNIT=ICHAN,FILE=outspec,FORM='BINARY',
     &  STATUS='UNKNOWN',ERR=9999)

	WRITE(ICHAN) ftflags,fversn
      WRITE(ICHAN)fexper,fexp,fnpts,ffirst,flast,fnsub,fxtype,fytype
	WRITE(ICHAN)fztype,fpost,fdate,fres,fsource,fpeakpt
	WRITE(ICHAN)(fspare(I),I=1,8)
	WRITE(ICHAN)fcmnt,fcatxt,flogoff,fmods,fprocs,flevel,fsampin
	WRITE(ICHAN)ffactor,fmethod,fzinc,fwplanes,fwinc,fwtype,freserv
	WRITE(ICHAN)SUBHDR

C  WRITE THE Y VALUES as REAL*4 numbers
      DO 50 I=1,fnpts
        WRITE(ICHAN) R(I)
  50  CONTINUE
C  If flogoff is not zero, write the log block

	IF(flogoff.NE.0)THEN
	  WRITE(ICHAN)LOGSIZD,LOGSIZM,LOGTXTO,LOGBINS,logpbins
	  
	      DO 55 I=1,NLGSPAR
	      WRITE(ICHAN) LOGSPAR(I)
   55       CONTINUE

	      DO 60 J=1,NLGDATA
	      WRITE(ICHAN)LOGtext(J)
   60       CONTINUE
   	ENDIF

C   *** FINISH UP AND RETURN NORMAL
      CALL PUTCHAN(ICHAN,IER)
      RETURN
C   ERROR HANDLING AND ABNORMAL RETURNS
 9999    STOP 'ERROR OPENING SPECTRA-CALC FILE IN WRITESC'
      END

	SUBROUTINE COMBDATE(fdate,IYR,IMON,IDAY,IHR,IMIN,D)
	INTEGER*2 IYR
	INTEGER*4 fdate,NREM
	INTEGER*1 imon,iday,ihr,imin
	CHARACTER*1 D
C  The year goes into bits 20 to 32, multiply IYR by 2**20=1048576
C  The month goes into bits 16 to 19,multiply IMON by 2**16=65536
C  The day goes into bits 11 to 15, multiply IDAY by 2**11=2048
C  The hour goes into bits 6 to 10, multiply IHR by 2**6=64
C  and add IMIN to the total.

C  IF D='C'  combine the components into fdate.
C  If D= anything except C,separate fdate into its components
	IF(D.EQ.'C')THEN
	fdate=IYR*1048576+IMON*65536+IDAY*2048+IHR*64+IMIN
	GO TO 20
	ELSE
	IYR=fdate/1048576
	NREM=fdate-1048576*IYR
	IMON=NREM/65536
	NREM=NREM-IMON*65536
	IDAY=NREM/2048
	NREM=NREM-IDAY*2048
	IHR=NREM/64
	IMIN=NREM-IHR*64
	ENDIF
   20	RETURN
	END

      SUBROUTINE GETNAM(PROMPT,filenam,IFLAG,IER)
      CHARACTER*12   filenam
      CHARACTER*72	PROMPT
      INTEGER*4         IER
C
C	THIS ROUTINE PROMPTS FOR A FILENAME AND CHECKS THAT IT DOES
C	EXIST. I NEED THIS ALL THE TIME.
C
C	THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C	CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C	WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C	CYBERNETICS HALO GRAPHICS.
C	WRITTEN AND MODIFIED BY KENNETH W. VAN EVERY
C	UC RIVERSIDE, CA.
C

      LOGICAL FEXIST
      ITR=1
      CALL LENSTR(PROMPT,LEN,IER)
3      WRITE(*,'(2(A)\)') PROMPT(1:LEN),' '
4      READ(*,'(A)',ERR=9001) filenam
C	AVOID ERROR WHEN A BAD NAME IS ENTERED
      INQUIRE(FILE=filenam,EXIST=FEXIST,ERR=9002,IOSTAT=IERR)
 
C	MAKE SURE THE FILE IS THERE.
      IF( IFLAG .EQ. 0 ) RETURN
      INQUIRE(FILE=filenam,EXIST=FEXIST)
      IF( .NOT. FEXIST ) THEN
            IER=0
            WRITE(*,*) ' GETNAME: FILE NOT FOUND - RETYPE.'
            ITR=ITR+1
            IF( ITR .GE.5 ) RETURN
            GO TO 3
      ENDIF
      RETURN
9001	CALL MSG('\AN ERROR WAS ENCOUNTERED READING\')
      CALL MSG('\THAT NAME,  PLEASE RE-ENTER OR\')
      CALL MSG('\CHOOSE ANOTHER NAME\')
      GO TO 3
9002	CALL MSG('\SOMETHING IS WRONG WITH THE FILENAME\')
      CALL MSG('\YOU ENTERED.  PLEASE CHECK FOR ERRORS\')
      CALL MSG('\OR ILLEGAL CHARACTERS\')
      GO TO 3
      END


      SUBROUTINE LENSTR(ASTR,LEN,IER)
      CHARACTER*100 ASTR
      INTEGER*4     IER
C
C	RETURNS THE LENGTH (LEN) OF A STRING ASTR WHICH HAS UNIQUE
C	DELIMITERS AT THE START AND END. DELIMITER MAY BE ANY
C	CHARACTER WHICH DOES NOT OCCUR WITHIN THE VALID STRING.
C	A \ CHARACTER IS RECOMMENDED AS CONVENTION BUT THE ROUTINE
C	DOES NOT CARE.
C
C	THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C	CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C	WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C	CYBERNETICS HALO GRAPHICS.
C	WRITTEN AND MODIFIED BY KENNETH W. VAN EVERY
C	UC RIVERSIDE, CA.
C
C
      IER=1
      DO 1 I=2,100
            IF( ASTR(I:I) .EQ. ASTR(1:1) ) THEN
                  LEN=I-1
                  RETURN
            ENDIF
1     CONTINUE
      IER=-1
      RETURN
      END


      SUBROUTINE MSG(ASTRING)
C
C	THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C	CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C	WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C	CYBERNETICS HALO GRAPHICS.
C	WRITTEN  BY  KENNETH W. VAN EVERY  UC RIVERSIDE, CA.
C
      CHARACTER*100 ASTRING
      INTEGER*4     IER
      CALL LENSTR(ASTRING,J,IER)
2     WRITE(*,'(1X,A)') ASTRING(2:J)
      RETURN
      END


      SUBROUTINE AHILO(ARA,ISP,IEP,RMX,RMN)
C
C   THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C   CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C   WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C   CYBERNETICS HALO GRAPHICS.
C   WRITTEN AND MODIFIED BY KENNETH W. VAN EVERY, U.C RIVERSIDE, CA.
C
C   AHILO
C   FINDS THE REAL MAX (RMX) AND REAL MIN (RMN) VALUES IN THE RANGE
C   BOUNDED BY ISP AND IEP, IN THE REAL ARRAY ARA.
C
      DIMENSION ARA(*)
      INTEGER*4  ISP,IEP,i
      RMX=ARA(ISP)
      RMN=ARA(ISP)
C   WRITE(*,*) 'ISP,IEP',ISP,IEP
      DO 10 I=ISP,IEP
      IF( ARA(I) .GT. RMX ) RMX=ARA(I)
      IF( ARA(I) .LT. RMN ) RMN=ARA(I)
10    CONTINUE
C   WRITE(*,*) 'RMX,RMN',RMX,RMN
      RETURN
      END


      SUBROUTINE GETCHAN(ICHAN,IER)
C
C   THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL ANALYSIS PACKAGE
C   CONVERTED TO RUN ON XXX/PC COMPATIBLE COMPUTERS. IT IS WRITTEN
C   WITH MICROSOFT FORTRAN 77 VERSION 4.0 AND USES THE MEDIA
C   CYBERNETICS HALO GRAPHICS.
C   WRITTEN AND MODIFIED BY KENNETH W. VAN EVERY, UC RIVERSIDE, CA.
C
C   THIS ROUTINE KEEPS TRACK OF A LIMITED NUMBER OF I/O
C   UNIT NUMBERS. ALTHOUGH NEITHER DOS OR MS-F77 SET THIS LIMIT
C   MOST SYSTEMS RUN WITH FILES=20.
C   THIS FEATURE WAS DESIRABLE FOR COMPATIBILITY WITH DG ROUTINES
C   AND FOR ALLOWING MANY MODULAR SUBPROGRAMS TO BE USED TOGETHER
C   WITHOUT WORRYING ABOUT WHO IS USING WHICH UNIT.
C   THIS ROUTINE IS COMPLEMENTED BY PUTCHAN WHICH IS USED TO
C   CLOSE A CHANNEL AND RETURN IS VALUE TO THE USABLE LIST.
C
C   TO USE THIS ROUTINE, CALL IT BEFORE A CHANNEL MUST
C   BE OPENED. IT RETURNS THE NUMBER OF A CHANNEL THAT
C   IS YET UNCLAIMED. THE CALLING ROUTINE CAN CHECK IER
C   TO MAKE SURE THAT NO ERRORS OCCURED. THIS CHANNEL NUMBER
C   MAY THEN BE OPENED AND USED AS NORMAL. WHEN THE PROGRAM
C   IS DONE WITH THE I/O UNIT IT SHOULD BE CLOSED WITH THE PUTCHAN
C   ROUTINE. THIS CLOSES THE CHANNEL AND RETURNS THE UNIT NUMBER TO
C   THE STACK. THIS ROUTINE WILL NOT RETURN THE NUMBERS OF THE
C   PRECONNECTED I/O CHANNELS (0,5,6). THE I/O UNIT=* IS ALWAYS
C   CONNECTED TO THE SCREEN/KEYBOARD COMBINATION.
C
C   THE SYSTEM BREAKS DOWN OF COURSE IF ROUTINES ARE OPENING
C   CHANNELS NOT TRACKED BY GETCHAN OR NOT RETURNING THE NUMBERS
C   TO PUTCHAN FOR CLOSURE.
C
C   The next statement is not needed in RWSPC. An essential difference
      integer*2 i,ioch,ichn,left,ichan
      INTEGER*4 IER
      COMMON /IOCHAN/ IOCH(20) ,ICHN(20),LEFT
      SAVE
C   WRITE(*,*) ' GETCHAN '
C   ASSUME NO ERRORS
      IER=1

      IF( LEFT .EQ. 0 ) THEN
            IER=13
            WRITE(*,*) 'GETCHAN: NO MORE I/O UNITS FREE. '
            RETURN
      ENDIF

      ICHAN=0
      DO 10 I=1,20
      IF(ICHN(I) .NE. 1 )THEN
            ICHAN=IOCH(I)
            LEFT=LEFT-1
            ICHN(I)=1
C   WRITE(*,*) 'LEAVING GETCHAN ' , ICHAN
            RETURN
      ENDIF

10    CONTINUE
      IER =13
      RETURN
      END


      SUBROUTINE PUTCHAN(ICHAN,IER)
C
C   THIS ROUTINE IS PART OF THE IMX LIKE SPECTRAL
C   PACKAGE ADAPTED TO RUN ON I?M COMPATIBLE PCS.
C
C   WRITTEN BY  KENNETH W. VAN EVERY, CHEM. DEPT., U.C. RIVERSIDE, CA.
C
C   THIS ROUTINE IS THE COUNTERPART OF GETCHAN
C   AND IS USED TO CLOSE A CHANNEL WITH THE
C   UNIT NUMBER PREVIOUSLY OBTAINED FROM GETCHAN.
C   SEE THE COMMENTS IN GETCHAN FOR FURTHER DETAILS.
C   The next statement is not needed in RWSPC. An essential difference
C
      integer*2 ioch,ichn,left,ichan
      INTEGER*4 ier
      COMMON /IOCHAN/ IOCH(20) ,ICHN(20) ,LEFT
C
C   THE ARRAY IOCH HAS THE CHANNEL NUMBERS WE ARE KEEPING TRACK OF
C   THE ARRAY ICHN CONTAINS ZEROS OR ONES TO INDICATE WHETHER
C   THE CORRESPONDING CHANNEL IN IOCH IS IN USE.
C   LEFT IS USED TO KEEP TRACK OF THE NUMBER OF CHANNELS
C   IN OUR LIST WHICH ARE STILL AVAILABLE.
C
      SAVE
C   WRITE(*,*) ' PUTCHAN ', ICHAN
      IER=1
      IF( ICHN(ICHAN) .EQ. 1) THEN
            CLOSE(ICHAN,IOSTAT=IERR)
            ICHN(ICHAN)=0
            IF(LEFT .LT. 20) LEFT=LEFT+1
            IF( IERR .NE. 0) IER=-1
C   WRITE(*,*) 'LEAVING PUTCHAN',IER
            RETURN
      ENDIF

      WRITE(*,*) 'PUTCHAN:  UNIT ', ICHAN , ' WAS NOT OPEN. '
      RETURN
      END

      BLOCK DATA IOC
C   The next statement is not needed in RWSPC. An essential difference
      integer*2 ioch,ichn,left
      COMMON /IOCHAN/ IOCH(20), ICHN(20) ,LEFT
C   THE CHANNELS WE KEEP TRACK OF.
      DATA IOCH/1,2,3,4,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22/
      DATA LEFT /20/
      END


      SUBROUTINE GSPCNAM(PROMPT,filenam,IFLAG,IER)
C	THIS ROUTINE IS SAME AS GETNAM EXCEPT ASSUMING SPC AS
C	EXTENSION OF THE FILENAME -- ZHANG, JULY 1990
      CHARACTER*12   filenam
      CHARACTER*72      PROMPT
      INTEGER*4         IER
      LOGICAL FEXIST
      ITR=1
      CALL LENSTR(PROMPT,LEN,IER)
3     WRITE(*,'(2(A)\)') PROMPT(1:LEN),' '
4     READ(*,'(A)',ERR=9001) filenam
      CALL LENSTR1(filenam,LEN,IER)
      LEN1=LEN+1
      LEN4=LEN+4
      filenam(LEN1:LEN4)='.SPC'
C	AVOID ERROR WHEN A BAD NAME IS ENTERED
      INQUIRE(FILE=filenam,EXIST=FEXIST,ERR=9002,IOSTAT=IERR)
 
C	MAKE SURE THE FILE IS THERE.
      IF( IFLAG .EQ. 0 ) RETURN
      INQUIRE(FILE=filenam,EXIST=FEXIST)
      IF( .NOT. FEXIST ) THEN
         IER=0
         WRITE(*,*) ' GETNAME: FILE NOT FOUND - RETYPE.'
         ITR=ITR+1
         IF( ITR .GE.5 ) RETURN
         GO TO 3
      ENDIF
      RETURN
9001  CALL MSG('\AN ERROR WAS ENCOUNTERED READING\')
      CALL MSG('\THAT NAME,  PLEASE RE-ENTER OR\')
      CALL MSG('\CHOOSE ANOTHER NAME\')
      GO TO 3
9002  CALL MSG('\SOMETHING IS WRONG WITH THE FILENAME\')
      CALL MSG('\YOU ENTERED.  PLEASE CHECK FOR ERRORS\')
      CALL MSG('\OR ILLEGAL CHARACTERS\')
      GO TO 3
      END


      SUBROUTINE LENSTR1(ASTR,LEN,IER)
      CHARACTER*100 ASTR
      INTEGER*4     IER
      IER=1
      DO 1 I=2,100
      IF( ASTR(I:I) .EQ.' ' ) THEN
         LEN=I-1
         RETURN
         ENDIF
1     CONTINUE
      IER=-1
      RETURN
      END
